<?php

// XML::Simple - an emulation of the Perl XML::Simple module written in php
//
// (C) Copyright 2007 B King
// This file is part of WACS.
//
// WACS is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
//
// WACS is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with WACS; if not, write to the Free Software Foundation, Inc.,
// 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
//

// based upon code found at http://www.php.net/manual/en/ref.xml.php

$depth=-1;  // we want to ignore the top level token

class XMLSimple {
   var $xml_obj = null;
   var $output = array();
   var $attrs;

   public function XMLSimple() {
       $this->xml_obj = xml_parser_create();
       xml_parser_set_option($this->xml_obj, XML_OPTION_CASE_FOLDING, 0);
       xml_parser_set_option($this->xml_obj, XML_OPTION_SKIP_WHITE, 1);
       xml_set_object($this->xml_obj,$this);
       xml_set_character_data_handler($this->xml_obj, 'XMLcontent');
       xml_set_element_handler($this->xml_obj, "XMLopentag", "XMLclosetag");
   }

   public function parse($XMLfile) {
       if (!($xmlfp = fopen($XMLfile, "r"))) {
	   // grumble
           die("XMLSimple: Cannot open XML file: $XMLfile");
           return false;
       }

	// gulp the whole thing at once
       while ($data = fread($xmlfp, filesize($XMLfile) ))
       {
           if (!xml_parse($this->xml_obj, $data, feof($xmlfp)))
	   {
               die(sprintf("XMLSimple: error: %s at line %d",
               xml_error_string(xml_get_error_code($this->xml_obj)),
               xml_get_current_line_number($this->xml_obj)));
               xml_parser_free($this->xml_obj);
           }
       }

	// all done
       return true;
   }

   function XMLopentag($parser, $name, $attribs){
	global $depth;
	global $top;
	global $middle;
	global $current;
	// print "depth $depth: name = $name, attribs = ".
	// var_dump($attribs)."\n";
	if( $depth == 2 )
	{
		$current = $name;
	}
	if( $depth == 1 )
	{
		$middle = $name;
		$current = $name;
	}
	if( $depth == 0 )
	{
		$top = $name;
		$current = $name;
	}
	$depth++;
   }

   function XMLcontent($parser, $data) {
        global $depth;
        global $top;
        global $middle;
        global $current;
       $data = preg_replace( '/^\n/', '', $data );
       $data = preg_replace( '/\n$/', '', $data );
       $data = preg_replace( '/^\s+/', '', $data );
       $data = preg_replace( '/\s+$/', '', $data );
       if(!empty($data) && $data!="\n")
       {
	   // valid data
	   // print "data is: ".$data."\n";
	   if( $depth == 0 )
	   {
		$this->output[$top] = $data;
		// print $top.":".$data;
	   }
	   if( $depth == 1 )
	   {
		$this->output[$top] = $data;
		// print $top.":".$data;
	   }
	   if( $depth == 2 )
	   {
		$this->output[$top][$middle] = $data;
		// print $top.":".$middle.":".$data;
	   }
	   if( $depth == 3 )
	   {
		$this->output[$top][$middle][$current] = $data;
		// print $top.":".$middle.":".$current.":".$data;
	   }
       }
   }

   function XMLclosetag($parser, $name){
	global $depth;
	$depth--;
   }
}

?>
